from __future__ import absolute_import
import time

from oops_timeline.hooks import flatten_timeline
from raven.contrib.django.raven_compat.middleware import wsgi
from raven.contrib.django.raven_compat.models import client


def get_timeline_context(environ):
    context = {}
    timeline = environ.get('timeline.timeline')
    if timeline:
        flatten_timeline(context, {'timeline': timeline})
    oops_id = environ.get('oops.report', {}).get('id')
    if oops_id:
        context['X-Oops-Id'] = oops_id
    return context


class RavenContextMiddleware(object):

    def process_exception(self, request, exception):
        extra = get_timeline_context(request.environ)
        client.extra_context(extra)


class SoftRequestTimeout(Exception):
    """Soft request timeout expired"""


class Sentry(wsgi.Sentry):

    def __init__(self, application, soft_start_timeout=None):
        super(Sentry, self).__init__(application)
        self.soft_start_timeout = soft_start_timeout

    def __call__(self, environ, start_response):
        if self.soft_start_timeout:
            start_time = time.time()

        def sentry_start_response(status, headers, exc_info=None):
            if self.soft_start_timeout:
                elapsed_time = time.time() - start_time
                if elapsed_time * 1000 > self.soft_start_timeout:
                    try:
                        raise SoftRequestTimeout(
                            "start_response over timeout %s." %
                            self.soft_start_timeout)
                    except SoftRequestTimeout:
                        self.handle_exception(environ)
            return start_response(status, headers, exc_info=exc_info)

        return super(Sentry, self).__call__(environ, sentry_start_response)

    def handle_exception(self, environ=None):
        extra = get_timeline_context(environ)
        self.client.extra_context(extra)
        return super(Sentry, self).handle_exception(environ)
