from django.test import TestCase

from canonical_raven.processors import (
    RemoveCookiesProcessor,
    SanitizeTimelineDjangoProcessor,
)


class RemoveCookiesProcessorTestCase(TestCase):

    def setUp(self):
        super(RemoveCookiesProcessorTestCase, self).setUp()
        client = object()
        self.processor = RemoveCookiesProcessor(client)

    def get_http_data(self):
        return {
            'request': {
                'cookies': {'foo': 'bar'},
                'headers': {'Cookie': 'foo=bar'},
            }
        }

    def test_remove_cookies_from_data(self):
        data = self.get_http_data()
        data['request']['cookies'] = 'foo'

        result = self.processor.process(data)

        self.assertIn('request', result)
        http = result['request']
        self.assertNotIn('cookies', http)

    def test_remove_cookies_from_headers(self):
        data = self.get_http_data()
        data['request']['headers']['Cookie'] = 'foo'

        result = self.processor.process(data)

        self.assertIn('request', result)
        http = result['request']
        self.assertIn('headers', http)
        self.assertNotIn('Cookie', http['headers'])


class SanitizeTimelineDjangoProcessorTestCase(TestCase):

    def setUp(self):
        super(SanitizeTimelineDjangoProcessorTestCase, self).setUp()
        client = object()
        self.processor = SanitizeTimelineDjangoProcessor(client)

    def get_extra_data(self, timeline=None):
        return {
            'extra': {
                'timeline': timeline or [],
            }
        }

    def assert_query_redacted(self, query, expected):
        timeline = [(0, 0, 'some-category', query, 'other')]
        data = self.get_extra_data(timeline=timeline)

        result = self.processor.process(data)

        self.assertIn('extra', result)
        extra = result['extra']
        self.assertIn('timeline', extra)
        expected_timeline = [
            (0, 0, 'some-category', expected, 'other')
        ]
        self.assertEqual(extra['timeline'], expected_timeline)

    def test_filter_session_query(self):
        query = 'select * from django_session'
        expected = 'select <session query redacted>'
        self.assert_query_redacted(query, expected)

    def test_filter_user_query(self):
        query = 'select * from auth_user'
        expected = 'select <user query redacted>'
        self.assert_query_redacted(query, expected)

    def test_keep_safe_queries(self):
        query = 'select * from some_table'
        self.assert_query_redacted(query, query)
